# SPDX-License-Identifier: CC0-1.0

{ testEnv, pkgs, depot, buildPackages, ... }:
with pkgs;

testEnv.mkTest {
  name = "driver_manager";
  meta.maintainers = with pkgs.stdenv.lib.maintainers; [ ehmry ];
  testEnv = { drivers = ./../compositions/pc-drivers.dhall; };
  testInputs = (map pkgs.genodeSources.depot [
    "acpi_drv"
    "ahci_drv"
    "boot_fb_drv"
    "driver_manager"
    "dynamic_rom"
    "input_filter"
    "intel_fb_drv"
    "platform_drv"
    "ps2_drv"
    "report_rom"
    "rom_reporter"
    "usb_drv"
    "vesa_drv"
  ]) ++ (map pkgs.genodeSources.make [ "test/driver_manager" ]);

  testScript = ''
    catch { exec dd if=/dev/zero of=hdd_disk.raw bs=1M count=32 }
    catch { exec ${buildPackages.e2fsprogs}/bin/mke2fs -F bin/hdd_disk.raw }
    run_genode_until {.*all expected devices present and accessible.*} 120
  '';
  testConfig = ./driver_manager.dhall;
  qemuArgs = [
    "-device ahci,id=ahci"
    "-drive id=hdd,file=hdd_disk.raw,format=raw,if=none"
    "-device ide-hd,drive=hdd,bus=ahci.1"
  ];
}
