# SPDX-License-Identifier: CC0-1.0

{ testEnv, pkgs, ... }:
with pkgs;

let
  mkTest' = { name, testConfig, testInputs ? [ ]
    , ... }@attrs:
    testEnv.mkTest (attrs // {
      name = "solo5-" + name;
      inherit testConfig;
      testInputs = [ pkgs.solo5 pkgs.solo5.tests ] ++ testInputs;
    });

  applyMkTest = x: {
    inherit (x) name;
    value = mkTest' x;
  };

  mkTests = testList: builtins.listToAttrs (map applyMkTest testList);

  genodeDepot = pkgs.genodeSources.depot;
  genodeMake = pkgs.genodeSources.make;

  tests = [
    {
      name = "multi";
      testConfig = "${./.}/solo5.dhall";
      testInputs = map genodeMake [ "app/ping" ] ++ (map genodeDepot [
        "ram_block"
        "nic_bridge"
        "nic_loopback"
        "sequence"
      ]);
    }

    {
      name = "ssp";
      testConfig = ./ssp.dhall;
      testScript = ''
        run_genode_until {Error: stack protector check failed} 30
      '';
    }

  ];

in mkTests tests
